# -*- coding: utf-8 -*-

"""log_helper.py is a helper module for logging in Python. It provides a set of functions and classes for logging in Python."""

__author__ = "atgczcl"
__email__ = "atgczcl@163.com"
# __all__ = ["log_helper"]
# 导入模块
# import log_helper
# from pack import *